@@ -74,6 +74,18 @@ void EditWidget::addPage(const QString& labelText, const QIcon& icon, QWidget* w
     m_ui->categoryList->addCategory(labelText, icon);
 }
 
+bool EditWidget::hasPage(QWidget* widget)
+{
+    for (int i = 0; i < m_ui->stackedWidget->count(); i++) {
+        auto* scrollArea = qobject_cast<QScrollArea*>(m_ui->stackedWidget->widget(i));
+        if (scrollArea && scrollArea->widget() == widget) {
+            return true;
+        }
+    }
+
+    return false;
+}
+
 void EditWidget::setPageHidden(QWidget* widget, bool hidden)
 {
     int index = -1;
